﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;
using System.Windows.Media;
using static Demo.Win32;

namespace Demo
{


    public class DpiHelper
    {
        private Window _wnd;

        private Window wnd
        {
            [MethodImpl(MethodImplOptions.Synchronized)]
            get
            {
                return _wnd;
            }

            [MethodImpl(MethodImplOptions.Synchronized)]
            set
            {
                if (_wnd != null)
                {
                    _wnd.Closing -= wnd_Closing;
                    _wnd.Loaded -= wnd_Loaded;
                }

                _wnd = value;
                if (_wnd != null)
                {
                    _wnd.Closing += wnd_Closing;
                    _wnd.Loaded += wnd_Loaded;
                }
            }
        }
        private HwndSource hSource;
        private HwndSourceHook hHook;

        private int dpiPrev;


        public DpiHelper(Window w)
        {
            if (!Library.IsHosted)
                throw new Exception("Hooker is intended for Hosted content only!");
            wnd = w;
        }

        private bool IsHooked
        {
            get
            {
                return hHook is not null;
            }
            set
            {
                switch (true)
                {
                    case object _ when value && !IsHooked:
                        {
                            hHook = HookProc;
                            hSource.AddHook(hHook);
                            break;
                        }
                    case object _ when !value && IsHooked:
                        {
                            hSource.RemoveHook(hHook);
                            hHook = null;
                            break;
                        }
                }
            }
        }





        private void wnd_Closing(object sender, CancelEventArgs e)
        {
            if (IsHooked)
                IsHooked = false;
            hSource = null;
            wnd = null;
        }

        private void wnd_Loaded(object sender, RoutedEventArgs e)
        {
            hSource = wnd.GetSource();
            IsHooked = true;
            ApplyDPI((int)GetDpiForWindow(wnd.Handle()));
        }

        private void ApplyDPI(int dpi)
        {
            dpiPrev = dpi;
            VisualTreeHelper.SetRootDpi(wnd, new DpiScale(dpi / 96d, dpi / 96d));
            uint dpiSys = GetDpiForSystem();
            var trfScale = new ScaleTransform(dpi / (double)dpiSys, dpi / (double)dpiSys);
            wnd.LayoutTransform = trfScale;
            wnd.GetChild().LayoutTransform = trfScale;
        }

        private IntPtr HookProc(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            switch (msg)
            {
                case 0x2E0: // WM_DPICHANGED
                    {
                        int dpi = wParam.ToInt32() & 0xFFFF; // LoWord
                        var rec = Marshal.PtrToStructure<RECT>(lParam);
                        // Prevent the newly resized window overflowing onto the original monitor, thus triggering a loop
                        if (dpi > dpiPrev)
                        {
                            SetWindowPos(hWnd, IntPtr.Zero, rec.Left, rec.Top, rec.Width(), rec.Height(), SWP.NOZORDER | SWP.FRAMECHANGED);
                            ApplyDPI(dpi);
                            Debug.Print("Hooker<< DPI_CHANGED");
                        }
                        else if (dpi < dpiPrev)
                        {
                            ApplyDPI(dpi);
                            SetWindowPos(hWnd, IntPtr.Zero, rec.Left, rec.Top, rec.Width(), rec.Height(), SWP.NOZORDER | SWP.FRAMECHANGED);
                            Debug.Print("Hooker<< DPI_CHANGED");
                        }

                        break;
                    }
            }

            return default;
        }
    }
}