﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows;
using System.Windows.Forms;
using Forms = System.Windows.Forms;
using System.Windows.Forms.Integration;
using System.Windows.Interop;
using static Demo.Win32;

namespace Demo
{

    /// <summary>
/// A Form, used to handle/wrangle inputfocus between Excel and Wpf
/// </summary>
    [DesignerCategory("")]
    public class FocusHelper : Form
    {

        private Window wnd;
        private HwndSource hSource;
        private HwndSourceHook hHook;

        public FocusHelper(Window w)
        {
            SuspendLayout();
            Text = "FocusHelper";
            FormBorderStyle = FormBorderStyle.None;
            StartPosition = FormStartPosition.Manual;
            /* TODO ERROR: Skipped IfDirectiveTrivia
            #If 0 Then
            *//* TODO ERROR: Skipped DisabledTextTrivia
                    'Visible
                    Location = New System.Drawing.Point(0, 0)
                    Size = New System.Drawing.Size(200, 50)
                    BackColor = System.Drawing.Color.Cyan
            *//* TODO ERROR: Skipped ElseDirectiveTrivia
            #Else
            */        // Visible but transparent & offscreen
            Location = new System.Drawing.Point(-32000, -32000);
            Size = new System.Drawing.Size(32, 32);
            ShowInTaskbar = false;
            ShowIcon = false;
            TransparencyKey = BackColor;
            /* TODO ERROR: Skipped EndIfDirectiveTrivia
            #End If
            */
            ResumeLayout();

            // Hook it up with Wpf
            ElementHost.EnableModelessKeyboardInterop(w);
            wnd = w;
            wnd.Loaded += Wnd_Loaded;
        }

        protected override CreateParams CreateParams
        {
            get
            {
                var p = base.CreateParams;
                p.Style = p.Style & ~0x10000; // WS_TABSTOP
                p.ExStyle = p.ExStyle | 0x80;     // WS_EX_TOOLWINDOW
                return p;
            }
        }

        public bool IsHooked
        {
            get
            {
                return hHook is not null;
            }
            set
            {
                switch (true)
                {
                    case object _ when value && hHook is null:
                        {
                            hHook = HookProc;
                            hSource.AddHook(hHook);
                            break;
                        }
                    case object _ when !value && hHook is not null:
                        {
                            hSource.RemoveHook(hHook);
                            hHook = null;
                            break;
                        }
                }
            }
        }

        private void Wnd_Loaded(object sender, RoutedEventArgs e)
        {
            hSource = wnd.GetSource();
            IsHooked = true;
        }

        [DebuggerStepThrough]
        private IntPtr HookProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            try
            {
                switch (msg)
                {
                    case 0x6: // WindowMessages.WM_ACTIVATE
                        {
                            int wLo = wParam.ToInt32() & 0xFFFF;
                            int wHi = wParam.ToInt32() >> 16 & 0xFFFF; // If <> 0 then Minimized

                            switch (wLo)
                            {
                                case 0: // deactivate
                                    {
                                        break;
                                    }
                                case 1: // activate
                                    {
                                        if (!DoFocus(false))
                                        {
                                            SetFocus(lParam);
                                            handled = true;
                                        }

                                        break;
                                    }
                                case 2: // clickactivate
                                    {
                                        if (!DoFocus(true))
                                        {
                                            SetFocus(lParam);
                                            handled = true;
                                        }

                                        break;
                                    }
                            }

                            break;
                        }
                }
            }

            catch (Exception ex)
            {
                Debug.Print($"FocusHelper:HookProc:Exception:{ex.Message}");
                IsHooked = false;
            }

            return default;
        }

        public bool DoFocus(bool click = false)
        {
            switch (true)
            {
                case object _ when wnd is null:
                    {
                        return false;
                    }
                case object _ when Library.XlsApp is null:
                    {
                        return false;
                    }
                case object _ when !wnd.IsVisible:
                    {
                        return false;
                    }
                case object _ when !wnd.IsEnabled:
                    {
                        return false;
                    }
                case object _ when wnd.WindowState == System.Windows.WindowState.Minimized:
                    {
                        return false;
                    }
            }


            try
            {
                var f = GetFocus();

                if (f.Equals(wnd.Handle()))
                {
                    return true;
                }
                else if (f.Equals(Handle))
                {
                    SetFocus(wnd.Handle());
                    return true;
                }
                else if (!Library.XlsApp.DoReady())
                {
                    return false;
                }
                else
                {
                    SetFocus(Handle);
                    SetFocus(wnd.Handle());
                    if (click)
                    {
                        Forms.Application.DoEvents();
                        SendClick();
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                Debug.Print($"FocMgr:DoFocus:Exception:{ex.Message}");
                return false;
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                IsHooked = false;
                hSource?.Dispose();
                wnd.Loaded -= Wnd_Loaded;
                hSource = null;
                wnd = null;

            }

            base.Dispose(disposing);

        }

    }
}