﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

namespace Demo.Primitives
{

    [DebuggerStepThrough]
    public abstract class ObservableBase : StandardOleMarshalObject, INotifyPropertyChanged
    {

        public event PropertyChangedEventHandler PropertyChanged;

        public void SetProp<T>(ref T @field, T value, [CallerMemberName] string propertyName = null)
        {
            if ((typeof(T).IsEnum || typeof(T).BaseType.Equals(typeof(ValueType))) && @field.Equals(value))
            {
            }
            // do nothing
            else if (EqualityComparer<T>.Default.Equals(@field, default) || !@field.Equals(value))
            {
                @field = value;
                OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
            }
        }

        public void Notify([CallerMemberName] string propertyName = null)
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            PropertyChanged?.Invoke(this, e);
        }
    }
}