﻿using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Media;

namespace Demo
{
    public class ViewModel : Primitives.ObservableBase
    {

        public ViewModel()
        {
            // for designer...
            Library.EnsureApp();
            Library.EnsureResources();
        }

        public void Refresh()
        {
            Notify(string.Empty);
            Debug.Print("ViewModel:Refresh");
        }

        public Window Wnd
        {
            get
            {
                return Application.Current?.MainWindow;
            }
        }

        public bool IsHosted
        {
            get
            {
                return Library.IsHosted;
            }
        }


        public bool IsWndTgt
        {
            get
            {
                return ApiWindow == WpfTarget;
            }
        }
        public bool IsVisWnd
        {
            get
            {
                return VisualWindow == ApiWindow;
            }
        }

        public string Awareness
        {
            get
            {
                return Win32.GetThreadDpiAwarenessContext().ToDescription();
            }
        }

        public double ApiSystem
        {
            get
            {
                return Win32.GetDpiForSystem() / 96d;
            }
        }

        public double WpfTarget
        {
            get
            {
                return (Wnd?.GetTarget()?.TransformToDevice.M11) ?? 0.987d;
            }
        }

        public double ApiWindow
        {
            get
            {

                return Wnd is null ? 0.987d : Win32.GetDpiForWindow(Wnd.Handle()) / 96d;
            }
        }

        public double VisualWindow
        {
            get
            {
                return Wnd is null ? 0.987d : VisualTreeHelper.GetDpi(Wnd).DpiScaleX;
            }
        }

        public double VisualChild
        {
            get
            {
                return Wnd?.GetChild() is null ? 0.987d : VisualTreeHelper.GetDpi(Wnd.GetChild()).DpiScaleX;
            }
        }

        public double TransformWindow
        {
            get
            {
                return (Wnd?.LayoutTransform?.Value.M11) ?? 0.987d;
            }
        }
        public double TransformChild
        {
            get
            {
                return (Wnd?.GetChild()?.LayoutTransform?.Value.M11) ?? 0.987d;
            }
        }




        public bool IsCustom
        {
            get
            {
                {
                    var withBlock = Application.Current?.Resources.MergedDictionaries;
                    bool ret = withBlock.Any(e => (e.Source?.OriginalString.Contains("StylesCustom")) ?? false);
                    return ret;
                }
            }
            set
            {
                if (value && !IsCustom)
                {
                    Library.LoadResources("StylesCustom", "DpiDemo.Lib");
                    Notify();
                }
                else if (!value && IsCustom)
                {
                    Library.UnloadResources("StylesCustom");
                    Notify();
                }
            }
        }

        // Property IsCustomAP As Boolean
        // Get
        // With Application.Current?.Resources.MergedDictionaries
        // Dim ret = .Any(Function(e) If(e.Source?.OriginalString.Contains("StylesBarCustom"), False))
        // Return ret
        // End With
        // End Get
        // Set
        // If Value AndAlso Not IsCustomAP Then
        // LoadResources("StylesBarCustom", "DpiDemo.LibAP")
        // Notify()
        // ElseIf Not Value AndAlso IsCustomAP Then
        // UnloadResources("StylesBarCustom")
        // Notify()
        // End If
        // End Set
        // End Property


    }
}