﻿using System;
using System.ComponentModel;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Text;

namespace Demo
{

    public class Win32
    {

        #region Wrappers
        public static string GetFocusClassName()
        {
            var f = GetFocus();
            if (f == IntPtr.Zero)
            {
                return "NUL";
            }
            else
            {
                string c = GetClassName(f).ToUpperInvariant();
                if (c.StartsWith("HWNDWRAPPER"))
                {
                    return "WND";
                }
                else if (c.Contains("FORMS10.WINDOW"))
                {
                    return "FRM";
                }
                else if (c.Equals("XLMAIN") || c.Equals("XLDESK") || c.StartsWith("EXCEL"))
                {
                    return c;
                }
                else
                {
                    // return root for the focused control
                    var r = GetAncestor(f, GetAncestorFlags.ROOT);
                    c = GetClassName(r).ToUpperInvariant();
                    return c.Substring(0, Math.Min(8, c.Length));
                }
            }

        }

        internal static string GetClassName(IntPtr hWnd)
        {
            var sb = new StringBuilder(255);
            GetClassName(hWnd, sb, sb.Capacity);
            return sb.ToString();
        }
        internal static void SendClick()
        {
            var inp = new INPUT[2];
            inp[0].type = 0;
            inp[0].u.mi.dwFlags = MouseEventFlags.LEFTDOWN;
            inp[1].type = 0;
            inp[1].u.mi.dwFlags = MouseEventFlags.LEFTUP;
            SendInput(2, inp, Marshal.SizeOf(typeof(INPUT)));
        }

        internal static void SendEsc()
        {
            const short VK_ESC = 0x1B;
            var inp = new INPUT[2];
            inp[0].type = 1;
            inp[0].u.ki.wVk = VK_ESC;
            inp[0].u.ki.dwFlags = 0;
            inp[1].type = 1;
            inp[1].u.ki.wVk = VK_ESC;
            inp[1].u.ki.dwFlags = 2;
            SendInput(2, inp, Marshal.SizeOf(typeof(INPUT)));
        }

        #endregion

        #region Imports ShCore (Win8.1)
        /// <summary>
        /// Win8.1 and up
        /// </summary>
        [DllImport("Shcore.dll", SetLastError = true)]
        public static extern int GetProcessDpiAwareness(IntPtr hProcess, out PROCESS_DPI_AWARENESS value);
        /// <summary>
        /// Win8.1 and up
        /// </summary>
        [DllImport("Shcore.dll", SetLastError = true)]
        public static extern int SetProcessDpiAwareness(PROCESS_DPI_AWARENESS dpiAwareness);
        /// <summary>
        /// Win8.1 and up
        /// </summary>
        [DllImport("Shcore.dll", SetLastError = true)]
        public static extern int GetDpiForMonitor(IntPtr hMonitor, MONITOR_DPI_TYPE dpiType, out uint dpiX, out uint dpiY);

        /// <summary>
        /// Note: Access description via extension.
        /// </summary>
        public enum PROCESS_DPI_AWARENESS
        {
            [Description("DpiUnaware")]
            PROCESS_DPI_UNAWARE = 0,
            [Description("DpiSystemAware")]
            PROCESS_SYSTEM_DPI_AWARE = 1,
            [Description("DpiPerMonitorAware")]
            PROCESS_PER_MONITOR_DPI_AWARE = 2
        }

        /// <summary>
        /// Note: Access description via extension.
        /// </summary>
        public enum DPI_AWARENESS
        {
            [Description("DpiInvalidAwareness")]
            DPI_AWARENESS_INVALID = -1,
            [Description("DpiUnaware")]
            DPI_AWARENESS_UNAWARE = 0,
            [Description("DpiSystemAware")]
            DPI_AWARENESS_SYSTEM_AWARE = 1,
            [Description("DpiPerMonitorAware")]
            DPI_AWARENESS_PER_MONITOR_AWARE = 2
        }
        public enum MONITOR_DPI_TYPE : uint
        {
            Default = 0U,
            EffectiveDpi = 0U,
            AngularDpi = 1U,
            RawDpi = 2U
        }
        #endregion

        #region Imports
        [DllImport("user32.dll", SetLastError = true)]
        public static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int x, int y, int cx, int cy, SWP uFlags);
        [DllImport("user32.dll")]
        public static extern IntPtr GetFocus();
        [DllImport("user32.dll")]
        public static extern IntPtr SetFocus(IntPtr hWnd);
        [DllImport("user32.dll", CharSet = CharSet.Unicode)]
        private static extern bool GetClassName(IntPtr hWnd, StringBuilder lpClassName, int nMaxCount);
        [DllImport("user32.dll", SetLastError = true)]
        private static extern IntPtr GetAncestor(IntPtr hWnd, GetAncestorFlags gaFlags);
        [DllImport("user32.dll", SetLastError = true)]
        private static extern int SendInput(int nInputs, INPUT[] pInputs, int cbSize);


        /// <summary>
        /// The DPI for the system. Depends on DPI_AWARENESS.
        /// </summary>
        [DllImport("User32.dll", SetLastError = true)]
        public static extern uint GetDpiForSystem();

        /// <summary>
        /// The DPI for the window. Depends on DPI_AWARENESS.
        /// </summary>
        [DllImport("User32.dll", SetLastError = true)]
        public static extern uint GetDpiForWindow(IntPtr hWnd);

        [DllImport("User32.dll", SetLastError = true)]
        public static extern bool IsValidDpiAwarenessContext(DPI_AWARENESS_CONTEXT value);
        [DllImport("User32.dll", SetLastError = true)]
        public static extern bool AreDpiAwarenessContextsEqual(DPI_AWARENESS_CONTEXT dpiContextA, DPI_AWARENESS_CONTEXT dpiContextB);

        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_AWARENESS_CONTEXT GetDpiAwarenessContextForProcess(IntPtr hProcess = default);
        [DllImport("User32.dll", SetLastError = true)]
        public static extern bool SetProcessDpiAwarenessContext(DPI_AWARENESS_CONTEXT dpiContext);

        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_AWARENESS_CONTEXT GetThreadDpiAwarenessContext();
        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_AWARENESS_CONTEXT SetThreadDpiAwarenessContext(DPI_AWARENESS_CONTEXT dpiContext);
        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_HOSTING_BEHAVIOR GetThreadDpiHostingBehavior();
        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_HOSTING_BEHAVIOR SetThreadDpiHostingBehavior(DPI_HOSTING_BEHAVIOR value);

        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_AWARENESS_CONTEXT GetWindowDpiAwarenessContext(IntPtr hWnd);
        [DllImport("User32.dll", SetLastError = true)]
        public static extern DPI_HOSTING_BEHAVIOR GetWindowDpiHostingBehavior(IntPtr hWnd);

        #endregion

        [StructLayout(LayoutKind.Sequential)]
        public struct RECT
        {
            public int Left, Top, Right, Bottom;
            
            public RECT(int left, int top, int right, int bottom)
            {
                Left = left;
                Top = top;
                Right = right;
                Bottom = bottom;
            }
            public readonly int Width()
            {
                return Right - Left;
            }
            public readonly int Height()
            {
                return Bottom - Top;
            }

            public override readonly string ToString()
            {
                return $"{Left,5},{Top,-5} {Right - Left,5}x{Bottom - Top,-5}";
            }
        }



        public enum DPI_AWARENESS_CONTEXT // From win10.1803
        {
            [Description("DAC_Invalid")]
            INVALID = 0,
            [Description("DAC_Unaware")]
            UNAWARE = -1,
            [Description("DAC_System")]
            SYSTEM_AWARE = -2,
            [Description("DAC_PerMonitor")]
            PER_MONITOR_AWARE = -3,
            [Description("DAC_PerMonitorV2")]
            PER_MONITOR_AWARE_V2 = -4,
            [Description("DAC_GDIScaled")]
            UNAWARE_GDISCALED = -5
        }

        public enum DPI_HOSTING_BEHAVIOR // From win10.1803
        {
            [Description("DHB_Invalid")]
            INVALID = -1,
            [Description("DHB_Default")]
            DEFAULT = 0,
            [Description("DHB_Mixed")]
            MIXED = 1
        }

        [Flags()]
        public enum SWP : uint
        {
            NOSIZE = 0x1U,
            NOMOVE = 0x2U,
            NOZORDER = 0x4U,
            NOREDRAW = 0x8U,
            NOACTIVATE = 0x10U,
            FRAMECHANGED = 0x20U,
            SHOWWINDOW = 0x40U,
            HIDEWINDOW = 0x80U,
            NOCOPYBITS = 0x100U,
            NOOWNERZORDER = 0x200U,
            NOSENDCHANGING = 0x400U,

            DEFERERASE = 0x2000U,
            ASYNCWINDOWPOS = 0x4000U,
            SHOWDESKTOP = 0x8000U
        }
        private struct INPUT
        {
            public int @type;
            public InputUnion u;
        }

        [StructLayout(LayoutKind.Explicit)]
        private struct InputUnion
        {
            [FieldOffset(0)]
            public MOUSEINPUT mi;
            [FieldOffset(0)]
            public KEYBDINPUT ki;
            [FieldOffset(0)]
            public HARDWAREINPUT hi;
        }

#pragma warning disable 0649
        private struct MOUSEINPUT
        {
            public int dx;
            public int dy;
            public int mouseData;
            public MouseEventFlags dwFlags;
            public int time;
            public IntPtr dwExtraInfo;
        }
        private struct KEYBDINPUT
        {
            public short wVk;
            public short wScan;
            public int dwFlags;
            public int time;
            public IntPtr dwExtraInfo;
        }
        private struct HARDWAREINPUT
        {
            public int uMsg;
            public short wParamL;
            public short wParamH;
        }
#pragma warning restore 0649

        [Flags()]
        private enum MouseEventFlags : uint
        {
            ABSOLUTE = 0x8000U,
            LEFTDOWN = 0x2U,
            LEFTUP = 0x4U,
            MIDDLEDOWN = 0x20U,
            MIDDLEUP = 0x40U,
            MOVE = 0x1U,
            RIGHTDOWN = 0x8U,
            RIGHTUP = 0x10U,
            XDOWN = 0x80U,
            XUP = 0x100U,
            WHEEL = 0x800U,
            HWHEEL = 0x1000U
        }

        private enum GetAncestorFlags : uint
        {
            PARENT = 1U,
            ROOT = 2U,
            ROOTOWNER = 3U
        }

    }
}